@@ -7,14 +7,16 @@ import functools
 import inspect
 import json
 import os
+import posixpath
 import threading
 import traceback
 from typing import Any, Callable, Coroutine, Dict, Iterable, List, Optional, Tuple, Union
+from urllib import parse
 
 from apscheduler.schedulers import asyncio as apscheduler_asyncio
 from apscheduler.triggers import cron as apscheduler_cron
 import fastapi
-from fastapi import staticfiles
+from fastapi import routing, staticfiles
 import typing_extensions
 from typing_extensions import Literal
 import uvicorn
@@ -96,14 +98,17 @@ class _InductorFunction:
                 return ret_val
         return wrapper
 
-    def fastapi_wrapper_ui_internal(self, env: environment.Env) -> Coroutine:
+    def fastapi_wrapper_ui_internal(
+        self,
+        app_env: environment.Env,
+    ) -> Coroutine:
         """Returns wrapped version of f for use as a UI-specific HTTP endpoint.
 
         (Specifically, for use as an Inductor-internal UI-specific FastAPI
         endpoint.)
 
         Args:
-            env: The environment in which this function is being called.
+            app_env: The environment in which this function is being called.
         """
         # Wrap f
         fn = self.fn
@@ -126,7 +131,7 @@ class _InductorFunction:
             # exception's traceback
             except Exception: # pylint: disable=broad-except
                 page = ui.Page("Exception")
-                if isinstance(env, environment.Local):
+                if isinstance(app_env, environment.Local):
                     page.heading("A Python exception was raised:")
                     page.code(traceback.format_exc(), lang="python")
                 else:
@@ -163,7 +168,10 @@ class _InductorFunction:
 class App:
     """Class whose instances represent Inductor apps."""
 
-    def __init__(self, env: Union[environment.Env, Dict[str, environment.Env]]):
+    def __init__(
+        self,
+        env: Union[environment.Env, Dict[str, environment.Env]],  # pylint: disable=redefined-outer-name
+    ):
         """Constructs a new App instance.
 
         Args:
@@ -275,6 +283,17 @@ class App:
                         del apps_table[self._env.app_name]
             # Retrieve app ID
             self._app_id = apps_table[self._env.app_name]["app_id"]
+            # Construct object store URL for this app
+            if self._env.object_store_url.startswith("file://"):
+                self._object_store_url = os.path.join(
+                    self._env.object_store_url, self._app_id)
+            elif self._env.object_store_url.startswith(("s3://", "gs://")):
+                self._object_store_url = posixpath.join(
+                    self._env.object_store_url, self._app_id)
+            else:
+                raise ValueError(
+                    "Unrecognized environment object_store_url prefix: " +
+                    self._env.object_store_url)
             # Initialize table storing state for unique id generation
             unique_id_state_table = table_util.table_from_url(
                 database_url=self._env.database_url,
@@ -294,24 +313,26 @@ class App:
             self.tables._up()  # pylint: disable=protected-access
             self._run_time = False
             # Create FastAPI app and add functions having an http_url_path
-            fastapi_app = fastapi.FastAPI()
-            fastapi_app.mount(
+            self._fastapi_app = fastapi.FastAPI()
+            self._fastapi_app.mount(
                 f"{_INTERNAL_HTTP_URL_PATH_PREFIX}/frontend",
                 staticfiles.StaticFiles(
                     directory=ui.frontend_assets_dir_path()),
                 name=f"{_INTERNAL_HTTP_URL_PATH_PREFIX}/frontend")
             for f in self._functions:
                 if f.http_url_path:
+                    module_qualname = util.module_qualname(f.fn)
                     # Add HTTP endpoint.
-                    getattr(fastapi_app, f.http_method)(
-                        f.http_url_path, response_model=None)(
-                            f.fastapi_wrapper())
+                    getattr(self._fastapi_app, f.http_method)(
+                        f.http_url_path,
+                        response_model=None,
+                        tags=[module_qualname])(f.fastapi_wrapper())
                     # Add UI HTTP endpoint for Inductor-internal use.
                     if not f.http_url_path.startswith("/"):
                         raise ValueError(
                             f"http_url_path ({f.http_url_path}) must "
                             "start with a slash.")
-                    fastapi_app.post(
+                    self._fastapi_app.post(
                         f"{_INTERNAL_HTTP_URL_PATH_PREFIX}/ui" +
                         f.http_url_path)(
                             f.fastapi_wrapper_ui_internal(self._env))
@@ -338,7 +359,7 @@ class App:
                         f.fn.exec_async,
                         trigger=apscheduler_cron.CronTrigger.from_crontab(
                             cron_string, timezone=f.schedule_timezone))
-            @fastapi_app.on_event("startup")
+            @self._fastapi_app.on_event("startup")
             async def start_scheduler():
                 scheduler.start()
             # Enter app's run-time context
@@ -349,7 +370,7 @@ class App:
                     f.fn()
             # Start FastAPI app (this call is blocking)
             uvicorn.run(
-                fastapi_app,
+                self._fastapi_app,
                 host=self._env.http_host,
                 port=self._env.http_port,
                 reload=False,
@@ -444,22 +465,25 @@ class App:
             internal_tables_instance = table_catalog.TableCatalog(
                 table_util.table_from_url(
                     database_url=self._env.database_url,
-                    table_name=f"inductor_app_{self._app_id}_catalog_internal_metadata",
+                    table_name=(f"inductor_app_{self._app_id}_"
+                                "catalog_internal_metadata"),
                     primary_key_column="id"),
                 table_util.table_from_url(
                     database_url=self._env.database_url,
-                    table_name=f"inductor_app_{self._app_id}_catalog_internal_name_to_id",
+                    table_name=(f"inductor_app_{self._app_id}_"
+                                "catalog_internal_name_to_id"),
                     primary_key_column="name"),
                 self._env,
                 self._run_time,
                 database_url,
                 self.unique_id,
-                physical_table_name_prefix=f"inductor_app_{self._app_id}_internal")
+                physical_table_name_prefix=(f"inductor_app_{self._app_id}_"
+                                            "internal"))
         else:
             raise TypeError(f"Unsupported env type ({type(self._env)})")
         if hasattr(self, "_internal_tables_instance_pre_run"):
-            internal_tables_instance._table_creation_instructions = (
-                self._internal_tables_instance_pre_run._table_creation_instructions)  # pylint: disable=line-too-long
+            internal_tables_instance._table_creation_instructions = (  # pylint: disable=protected-access
+                self._internal_tables_instance_pre_run._table_creation_instructions)  # pylint: disable=line-too-long,protected-access
         return internal_tables_instance
 
     @property
@@ -505,11 +529,13 @@ class App:
             tables_instance = table_catalog.TableCatalog(
                 table_util.table_from_url(
                     database_url=self._env.database_url,
-                    table_name=f"inductor_app_{self._app_id}_catalog_app_metadata",
+                    table_name=(f"inductor_app_{self._app_id}_"
+                                "catalog_app_metadata"),
                     primary_key_column="id"),
                 table_util.table_from_url(
                     database_url=self._env.database_url,
-                    table_name=f"inductor_app_{self._app_id}_catalog_app_name_to_id",
+                    table_name=(f"inductor_app_{self._app_id}_"
+                                "catalog_app_name_to_id"),
                     primary_key_column="name"),
                 self._env,
                 self._run_time,
@@ -520,8 +546,8 @@ class App:
         else:
             raise TypeError(f"Unsupported env type ({type(self._env)})")
         if hasattr(self, "_tables_instance_pre_run"):
-            tables_instance._table_creation_instructions = (
-                self._tables_instance_pre_run._table_creation_instructions)
+            tables_instance._table_creation_instructions = (  # pylint: disable=protected-access
+                self._tables_instance_pre_run._table_creation_instructions)  # pylint: disable=protected-access
         return tables_instance
 
     @property
@@ -533,24 +559,32 @@ class App:
                 self._internal_tables["ml_model_metadata"],
                 self._unique_id_gen)
         elif isinstance(self._env, environment.Server):
-            if self._env.object_store_url.startswith("file://"):
+            if self._object_store_url.startswith("file://"):
                 return model_store.LocalModelStore(
                     os.path.join(
-                        self._env.object_store_url[len("file://"):],
+                        self._object_store_url[len("file://"):],
                         "ml",
                         "models"),
                     self._internal_tables["ml_model_metadata"],
                     self._unique_id_gen)
-            elif self._env.object_store_url.startswith("s3://"):
-                s3_url = self._env.object_store_url
+            elif self._object_store_url.startswith("s3://"):
+                s3_url = self._object_store_url
                 if not s3_url.endswith("/"):
                     s3_url += "/"
                 return model_store.S3ModelStore(
                     s3_url + "ml/models/",
                     self._internal_tables["ml_model_metadata"],
                     self._unique_id_gen)
+            elif self._object_store_url.startswith("gs://"):
+                gcs_url = self._object_store_url
+                if not gcs_url.endswith("/"):
+                    gcs_url += "/"
+                return model_store.GCSModelStore(
+                    gcs_url + "ml/models/",
+                    self._internal_tables["ml_model_metadata"],
+                    self._unique_id_gen)
             else:
-                raise RuntimeError(
+                raise ValueError(
                     "Unrecognized environment object_store_url prefix: " +
                     self._env.object_store_url)
         else:
@@ -626,6 +660,37 @@ class App:
             def fn_async_wrapper(*args, **kwargs):
                 return fn(*args, **kwargs)
             fn._fn_async = fn_async_wrapper  # pylint: disable=protected-access
+            # Add fn.url_path() if http_url_path is present
+            if http_url_path is not None:
+                def fn_url_path_wrapper(*args, **kwargs):
+                    fn_name = fn.__name__
+                    fn_args = dict(
+                        inspect.signature(fn).bind_partial(*args,
+                                                           **kwargs).arguments)
+                    module_qualname = util.module_qualname(fn)
+                    route = next(route for route in self._fastapi_app.routes
+                                 if route.name == fn_name and
+                                 module_qualname in route.tags)
+                    if not isinstance(route, routing.APIRoute):
+                        raise TypeError(
+                            f"{fn_name} route is not instance of APIRoute")
+                    path_params = {}
+                    query_params = {}
+                    for param in route.dependant.path_params:
+                        value = fn_args.get(param.name)
+                        if value is None:
+                            raise ValueError(
+                                f"missing a required argument: '{param.name}'")
+                        path_params[param.name] = value
+                    for param in route.dependant.query_params:
+                        value = fn_args.get(param.name)
+                        if value is not None:
+                            query_params[param.name] = value
+                    url = route.url_path_for(fn_name, **path_params)
+                    parsed = list(parse.urlparse(url))
+                    parsed[4] = parse.urlencode(query_params)
+                    return parse.urlunparse(parsed)
+                fn.url_path = fn_url_path_wrapper
             # Register fn with this app
             self._functions.append(_InductorFunction(
                 fn=fn,
@@ -662,15 +727,21 @@ class App:
         self._navbar_args = {
             "logo": logo, "left": left, "center": center, "right": right}
 
-    def Page(self, title: str) -> ui.Page:
+    def Page(  # pylint: disable=invalid-name
+        self,
+        title: str,
+        *,
+        vertical_align: Literal["top", "center"] = "top") -> ui.Page:
         """Returns a new web page container.
 
         See ui.Page for more information on how to use the returned object.
 
         Args:
             title: The page title to be displayed in the browser.
+            vertical_align: The vertical alignment for content added to the
+                page body.
         """
-        p = ui.Page(title)
+        p = ui.Page(title, vertical_align=vertical_align)
         if hasattr(self, "_navbar_args"):
             p.navbar(**self._navbar_args)
         return p
