@@ -113,33 +113,52 @@ class PostgresqlView(table.Table):
         """See base class."""
         query_string, query_values = self._query_string()
         rows = []
-        # pylint: disable-next=protected-access
-        with self._parent._connection.cursor() as cursor:
-            cursor.execute(query_string, query_values)
-            row_dict = cursor.fetchone()
-            while row_dict is not None:
-                # pylint: disable-next=protected-access
-                if self._parent._auto_ddl:
-                    row_dict = _clean_row_dict(row_dict)
-                row_dict = _standardize_output(row_dict)
-                rows.append(row_dict)
+        try:
+            with self._parent._connection.cursor() as cursor:  # pylint: disable=protected-access
+                cursor.execute(query_string, query_values)
                 row_dict = cursor.fetchone()
-        self._parent._connection.commit()
+                while row_dict is not None:
+                    if self._parent._auto_ddl:  # pylint: disable=protected-access
+                        row_dict = _clean_row_dict(row_dict)
+                    row_dict = _standardize_output(row_dict)
+                    rows.append(row_dict)
+                    row_dict = cursor.fetchone()
+            self._parent._connection.commit()  # pylint: disable=protected-access
+        except psycopg2.Error as error:
+            self._parent._connection.rollback()  # pylint: disable=protected-access
+            with self._parent._connection.cursor() as cursor:  # pylint: disable=protected-access
+                cursor.execute(
+                    f"SELECT COUNT(*) AS c FROM {self._parent._table_name}")  # pylint: disable=protected-access
+                total_num_rows = cursor.fetchone()["c"]
+            self._parent._connection.commit()  # pylint: disable=protected-access
+            if total_num_rows == 0:
+                rows = []
+            else:
+                raise error
         return rows.__iter__()
 
     def first_row(self) -> Optional[table.Row]:
         """See base class."""
         query_string, query_values = self._query_string()
-        # pylint: disable-next=protected-access
-        with self._parent._connection.cursor() as cursor:
-            cursor.execute(
-                f"SELECT * FROM ({query_string}) AS subquery LIMIT 1",
-                query_values)
-            row_dict = cursor.fetchone()
-        self._parent._connection.commit() # pylint: disable=protected-access
+        try:
+            with self._parent._connection.cursor() as cursor:  # pylint: disable=protected-access
+                cursor.execute(
+                    f"SELECT * FROM ({query_string}) AS subquery LIMIT 1",
+                    query_values)
+                row_dict = cursor.fetchone()
+            self._parent._connection.commit()  # pylint: disable=protected-access
+        except psycopg2.Error as error:
+            self._parent._connection.rollback()  # pylint: disable=protected-access
+            with self._parent._connection.cursor() as cursor:  # pylint: disable=protected-access
+                cursor.execute(
+                    f"SELECT COUNT(*) AS c FROM {self._parent._table_name}")  # pylint: disable=protected-access
+                total_num_rows = cursor.fetchone()["c"]
+            self._parent._connection.commit()  # pylint: disable=protected-access
+            if total_num_rows == 0:
+                return None
+            raise error
         if row_dict is not None:
-            # pylint: disable-next=protected-access
-            if self._parent._auto_ddl:
+            if self._parent._auto_ddl:  # pylint: disable=protected-access
                 row_dict = _clean_row_dict(row_dict)
             row_dict = _standardize_output(row_dict)
         return row_dict
@@ -534,7 +553,7 @@ class PostgresqlKeyedTable(PostgresqlTable, table.WithPrimaryKey):
                 c = cursor.fetchone()["c"]
             self._connection.commit()
             return c > 0
-        except psycopg2.Error:
+        except psycopg2.Error as error:
             self._connection.rollback()
             with self._connection.cursor() as cursor:
                 cursor.execute(f"SELECT COUNT(*) AS c FROM {self._table_name}")
@@ -543,7 +562,7 @@ class PostgresqlKeyedTable(PostgresqlTable, table.WithPrimaryKey):
             if total_num_rows == 0:
                 return False
             else:
-                raise
+                raise error
 
     def set(
         self, key: Any, row: table.Row, skip_if_exists: bool = False) -> bool:
