@@ -108,30 +108,56 @@ class SnowflakeView(table.Table):
         """See base class."""
         query_string, query_values = self._query_string()
         rows = []
-        with self._parent._connection.cursor( # pylint: disable=protected-access
-            connector.DictCursor) as cursor:
-            cursor.execute(query_string, query_values)
-            row_dict = cursor.fetchone()
-            while row_dict is not None:
-                # pylint: disable-next=protected-access
-                if self._parent._auto_ddl:
-                    row_dict = _clean_row_dict(row_dict)
-                row_dict = _standardize_output(row_dict)
-                rows.append(row_dict)
+        try:
+            # pylint: disable-next=protected-access
+            with self._parent._connection.cursor(
+                connector.DictCursor) as cursor:
+                cursor.execute(query_string, query_values)
                 row_dict = cursor.fetchone()
-        self._parent._connection.commit()
+                while row_dict is not None:
+                    # pylint: disable-next=protected-access
+                    if self._parent._auto_ddl:
+                        row_dict = _clean_row_dict(row_dict)
+                    row_dict = _standardize_output(row_dict)
+                    rows.append(row_dict)
+                    row_dict = cursor.fetchone()
+            self._parent._connection.commit()
+        except connector.errors.ProgrammingError as error:
+            self._parent._connection.rollback()  # pylint: disable=protected-access
+            with self._parent._connection.cursor() as cursor:  # pylint: disable=protected-access
+                cursor.execute(
+                    f"SELECT COUNT(*) FROM \"{self._parent._table_name}\"")  # pylint: disable=protected-access
+                row_count = cursor.fetchone()[0]
+            self._parent._connection.commit()  # pylint: disable=protected-access
+            if row_count == 0:
+                rows = []
+            else:
+                raise error
         return rows.__iter__()
 
     def first_row(self) -> Optional[table.Row]:
         """See base class."""
-        # pylint: disable-next=protected-access
-        with self._parent._connection.cursor(connector.DictCursor) as cursor:
-            query_string, query_values = self._query_string()
-            cursor.execute(
-                f"SELECT * FROM ({query_string}) AS subquery LIMIT 1",
-                query_values)
-            row_dict = cursor.fetchone()
-        self._parent._connection.commit()  # pylint: disable=protected-access
+        try:
+            # pylint: disable-next=protected-access
+            with self._parent._connection.cursor(
+                connector.DictCursor) as cursor:
+                query_string, query_values = self._query_string()
+                cursor.execute(
+                    f"SELECT * FROM ({query_string}) AS subquery LIMIT 1",
+                    query_values)
+                row_dict = cursor.fetchone()
+            # pylint: disable-next=protected-access
+            self._parent._connection.commit()
+        except connector.errors.ProgrammingError as error:
+            self._parent._connection.rollback()  # pylint: disable=protected-access
+            with self._parent._connection.cursor() as cursor:  # pylint: disable=protected-access
+                cursor.execute(
+                    f"SELECT COUNT(*) FROM \"{self._parent._table_name}\"")  # pylint: disable=protected-access
+                row_count = cursor.fetchone()[0]
+            self._parent._connection.commit()  # pylint: disable=protected-access
+            if row_count == 0:
+                return None
+            raise error
         if row_dict is not None:
             # pylint: disable-next=protected-access
             if self._parent._auto_ddl:
