@@ -97,10 +97,22 @@ class SqliteView(table.Table):
         """See base class."""
         query_string, query_values = self._query_string()
         rows = []
-        for row in self._parent._connection.execute(query_string, query_values):
-            # pylint: disable-next=protected-access
-            row_dict = _row_to_row_dict(self._parent._auto_ddl, row)
-            rows.append(_standardize_output(row_dict))
+        try:
+            for row in self._parent._connection.execute(
+                query_string, query_values):
+                # pylint: disable-next=protected-access
+                row_dict = _row_to_row_dict(self._parent._auto_ddl, row)
+                rows.append(_standardize_output(row_dict))
+        except sqlite3.Error as error:
+            self._parent._connection.rollback()  # pylint: disable=protected-access
+            total_num_rows = self._parent._connection.execute(  # pylint: disable=protected-access
+                f"SELECT COUNT(*) FROM {self._parent._table_name}"  # pylint: disable=protected-access
+            ).fetchone()[0]
+            self._parent._connection.commit()  # pylint: disable=protected-access
+            if total_num_rows == 0:
+                rows = []
+            else:
+                raise error
         return rows.__iter__()
 
     def first_row(self) -> Optional[table.Row]:
@@ -109,12 +121,22 @@ class SqliteView(table.Table):
         query_string = (
             f"WITH view_rows AS ({query_string}) "
             f"SELECT * FROM view_rows LIMIT 1")
-        # pylint: disable-next=protected-access
-        for row in self._parent._connection.execute(query_string, query_values):
+        try:
             # pylint: disable-next=protected-access
-            row_dict = _row_to_row_dict(self._parent._auto_ddl, row)
-            row_dict = _standardize_output(row_dict)
-            return row_dict
+            for row in self._parent._connection.execute(
+                query_string, query_values):
+                # pylint: disable-next=protected-access
+                row_dict = _row_to_row_dict(self._parent._auto_ddl, row)
+                row_dict = _standardize_output(row_dict)
+                return row_dict
+        except sqlite3.Error as error:
+            self._parent._connection.rollback()  # pylint: disable=protected-access
+            total_num_rows = self._parent._connection.execute(  # pylint: disable=protected-access
+                f"SELECT COUNT(*) FROM {self._parent._table_name}"  # pylint: disable=protected-access
+            ).fetchone()[0]
+            self._parent._connection.commit()  # pylint: disable=protected-access
+            if total_num_rows != 0:
+                raise error
         return None
 
     @property
@@ -415,14 +437,14 @@ class SqliteKeyedTable(SqliteTable, table.WithPrimaryKey):
             return self._connection.execute(
                 f"SELECT COUNT(*) FROM {self._table_name} "
                 f"WHERE {self._primary_key_column}=?", (key,)).fetchone()[0] > 0
-        except sqlite3.Error:
+        except sqlite3.Error as error:
             self._connection.rollback()
             total_num_rows = self._connection.execute(
                 f"SELECT COUNT(*) FROM {self._table_name}").fetchone()[0]
             if total_num_rows == 0:
                 return False
             else:
-                raise
+                raise error
 
     def set(
         self, key: Any, row: table.Row, skip_if_exists: bool = False) -> bool:
