@@ -4,9 +4,8 @@
 
 import importlib
 import json
-import os
 import sqlite3
-from typing import Any, Callable, Dict, List, Optional, Tuple, Union
+from typing import Any, Callable, Dict, List, Optional, Tuple
 
 import boto3
 import psycopg2
@@ -218,9 +217,9 @@ class TableCatalog:
                 should be accelerated (generally via creation of indexes).
 
         Raises:
-            RuntimeError if a table having given name already exists but does
-            not have the characteristics specified by this method call's
-            arguments.
+            RuntimeError: if a table having given name already exists but does
+                not have the characteristics specified by this method call's
+                arguments.
         """
         table_spec = locals().copy()
         del table_spec["self"]
@@ -286,9 +285,9 @@ class TableCatalog:
                 should be accelerated (generally via creation of indexes).
 
         Raises:
-            RuntimeError if a table having given name already exists but does
-            not have the characteristics specified by this method call's
-            arguments.
+            RuntimeError: if a table having given name already exists but does
+                not have the characteristics specified by this method call's
+                arguments.
         """
         table_spec = locals().copy()
         del table_spec["self"]
@@ -330,7 +329,7 @@ class TableCatalog:
         self,
         name: str,
         sqlite_file_path: str,
-        sqlite_table_name: str,
+        sqlite_table_name: Optional[str] = None,
         primary_key_column: Optional[str] = None):
         """Adds a reference in this app to an existing sqlite table.
 
@@ -344,10 +343,12 @@ class TableCatalog:
             sqlite_file_path: Path to file giving sqlite database containing
                 the existing table to be added.
             sqlite_table_name: Name of existing table in file given by
-                sqlite_file_path.
+                sqlite_file_path. If not provided, the value of the name
+                argument is used.
             primary_key_column: If non-None, then the table will be treated as
                 being keyed by the column having name given by this parameter.
-                If None:
+                If None, then
+
                 - If the existing table has exactly one primary key column, then
                   the table will be treated as being keyed on that column.
                 - Otherwise, if the existing table has no primary key column
@@ -356,10 +357,12 @@ class TableCatalog:
                 - Otherwise, the table will not be treated as being keyed.
 
         Raises:
-            RuntimeError if a table having given name already exists but does
-            not have the characteristics specified by this method call's
-            arguments.
+            RuntimeError: if a table having given name already exists but does
+                not have the characteristics specified by this method call's
+                arguments.
         """
+        if sqlite_table_name is None:
+            sqlite_table_name = name
         table_spec = locals().copy()
         del table_spec["self"]
         if not self._run_time:
@@ -414,7 +417,7 @@ class TableCatalog:
         mysql_host: str,
         mysql_user: str,
         mysql_database: str,
-        mysql_table_name: str,
+        mysql_table_name: Optional[str] = None,
         primary_key_column: Optional[str] = None,
         mysql_port: Optional[int] = None,
         mysql_password: Optional[str] = None):
@@ -432,10 +435,12 @@ class TableCatalog:
             mysql_user: MySQL database server username.
             mysql_database: Name of MySQL database containing existing table.
             mysql_table_name: Name of existing table in database given by
-                preceding arguments.
+                preceding arguments. If not provided, the value of the name
+                argument is used.
             primary_key_column: If non-None, then the table will be treated as
                 being keyed by the column having name given by this parameter.
-                If None:
+                If None, then
+
                 - If the existing table has exactly one primary key column, then
                   the table will be treated as being keyed on that column.
                 - Otherwise, if the existing table has no primary key column
@@ -448,10 +453,13 @@ class TableCatalog:
                 password must be provided by secrets file.
 
         Raises:
-            RuntimeError if a table having given name already exists but does
-            not have the characteristics specified by this method call's
-            arguments.
+            RuntimeError: if a table having given name already exists but does
+                not have the characteristics specified by this method call's
+                arguments.
         """
+        if mysql_table_name is None:
+            mysql_table_name = name
+
         table_spec = locals().copy()
         del table_spec["self"]
         if not self._run_time:
@@ -529,7 +537,7 @@ class TableCatalog:
     def add_existing_dynamodb_table(
         self,
         name: str,
-        dynamodb_table_name: str,
+        dynamodb_table_name: Optional[str] = None,
         dynamodb_endpoint_url: Optional[str] = None):
         """Adds a reference in this app to an existing DynamoDB table.
 
@@ -540,7 +548,8 @@ class TableCatalog:
         Args:
             name: Name in this app under which to add a reference to the
                 specified existing DynamoDB table.
-            dynamodb_table_name: Name of existing DynamoDB table.
+            dynamodb_table_name: Name of existing DynamoDB table. If not
+                provided, the value of the name argument is used.
             dynamodb_endpoint_url: Optional Endpoint URL to be used to
                 communicate with the DynamoDB service. Intended to be used
                 to connect with a local DynamoDB instance for development
@@ -548,10 +557,12 @@ class TableCatalog:
                 if utilizing an actual AWS account.
 
         Raises:
-            RuntimeError if a table having given name already exists but does
-            not have the characteristics specified by this method call's
-            arguments.
+            RuntimeError: if a table having given name already exists but does
+                not have the characteristics specified by this method call's
+                arguments.
         """
+        if dynamodb_table_name is None:
+            dynamodb_table_name = name
         table_spec = locals().copy()
         del table_spec["self"]
         if not self._run_time:
@@ -597,7 +608,7 @@ class TableCatalog:
         postgresql_host: str,
         postgresql_user: str,
         postgresql_database: str,
-        postgresql_table_name: str,
+        postgresql_table_name: Optional[str] = None,
         primary_key_column: Optional[str] = None,
         postgresql_port: Optional[int] = None,
         postgresql_password: Optional[str] = None):
@@ -616,10 +627,12 @@ class TableCatalog:
             postgresql_database: Name of PostgreSQL database containing
                 existing table.
             postgresql_table_name: Name of existing table in database given by
-                preceding arguments.
+                preceding arguments. If not provided, the value of the name
+                argument is used.
             primary_key_column: If non-None, then the table will be treated as
                 being keyed by the column having name given by this parameter.
-                If None:
+                If None, then
+
                 - If the existing table has exactly one primary key column,
                   then the table will be treated as being keyed on that column.
                 - Otherwise, if the existing table has no primary key column
@@ -632,10 +645,13 @@ class TableCatalog:
                 provided, password must be provided by secrets file.
 
         Raises:
-            RuntimeError if a table having given name already exists but does
-            not have the characteristics specified by this method call's
-            arguments.
+            RuntimeError: if a table having given name already exists but does
+                not have the characteristics specified by this method call's
+                arguments.
         """
+        if postgresql_table_name is None:
+            postgresql_table_name = name
+
         table_spec = locals().copy()
         del table_spec["self"]
         if not self._run_time:
@@ -725,7 +741,7 @@ class TableCatalog:
         snowflake_database: str,
         snowflake_schema: str,
         snowflake_warehouse: str,
-        snowflake_table_name: str,
+        snowflake_table_name: Optional[str] = None,
         snowflake_password: Optional[str] = None):
         """Adds a reference in this app to an existing Snowflake table.
 
@@ -747,19 +763,24 @@ class TableCatalog:
             snowflake_warehouse: Snowflake warehouse name to use with this
                 table.
             snowflake_table_name: Name of existing table in database given by
-                preceding arguments.
+                preceding arguments. If not provided, the value of the name
+                argument is used.
             snowflake_password: Optionally, password for user.  If not
                 provided, password must be provided by secrets file.
 
         Raises:
-            RuntimeError if a table having given name already exists but does
-            not have the characteristics specified by this method call's
-            arguments.
+            RuntimeError: if a table having given name already exists but does
+                not have the characteristics specified by this method call's
+                arguments.
         """
         from inductor.data.table import snowflake  # pylint: disable=import-outside-toplevel
 
+        if snowflake_table_name is None:
+            snowflake_table_name = name
+
         table_spec = locals().copy()
         del table_spec["self"]
+        del table_spec["snowflake"]
 
         if not self._run_time:
             self._table_creation_instructions.append(
@@ -801,3 +822,69 @@ class TableCatalog:
                 raise RuntimeError(
                     f"Table {name} already exists but has different "
                     "table_spec.")
+
+    def add_existing_bigquery_table(
+        self,
+        name: str,
+        bigquery_dataset_name: str,
+        bigquery_table_name: Optional[str] = None,
+        gcp_project_id: Optional[str] = None):
+        """Adds a reference in this app to an existing BigQuery table.
+
+        If using this method, Inductor must have been installed with
+        BigQuery support enabled, as follows:
+        `pip install --upgrade inductor[bigquery]`
+
+        If a table is already referenced in this app under the given name, then
+        it must have the characteristics specified by this method call's
+        arguments; otherwise, this method raises an exception.
+
+        Args:
+            name: Name in this app under which to add a reference to the
+                specified existing BigQuery table.
+            bigquery_dataset_name: Name of BigQuery dataset containing the
+                table.
+            bigquery_table_name: Name of existing BigQuery table.  If not
+                provided, the value of the `name` argument is used.
+            gcp_project_id: Optionally, the ID of the Google Cloud project
+                containing the BigQuery table.  If None, then will be inferred
+                from the environment in which this code is running (by Google's
+                BigQuery Python library).
+
+        Raises:
+            RuntimeError: if a table having given name already exists but does
+                not have the characteristics specified by this method call's
+                arguments.
+        """
+        from inductor.data.table import bigquery  # pylint: disable=import-outside-toplevel
+
+        if bigquery_table_name is None:
+            bigquery_table_name = name
+
+        table_spec = locals().copy()
+        del table_spec["self"]
+        del table_spec["bigquery"]
+
+        if not self._run_time:
+            self._table_creation_instructions.append(
+                (self.add_existing_bigquery_table.__name__, table_spec))
+            return
+
+        if name in self:
+            if self.table_spec(name) != table_spec:
+                raise RuntimeError(
+                    f"Table {name} already exists but has different "
+                    "table_spec.")
+        else:
+            t = bigquery.BigQueryTable(
+                dataset_name=bigquery_dataset_name,
+                table_name=bigquery_table_name,
+                project_id=gcp_project_id,
+                auto_ddl=False)
+            already_exists = not self._add_table(
+                name, t, table_spec, skip_if_exists=True, existing_table=True)
+            if (already_exists and
+                self.table_spec(name) != table_spec):
+                raise RuntimeError(
+                    f"Table {name} already exists but has different "
+                    "table_spec.")
